import _find from 'lodash/find';
import _isArray from 'lodash/isArray';
import _isString from 'lodash/isString';
import _isNumber from 'lodash/isNumber';
import _escapeRegExp from 'lodash/escapeRegExp';


const operators = {
    number: [
        {
            code: '<',
            title: '<',
            isValue: true,
            valueType: 'number',
            evaluate(dataConnector, value) {
                if (!_isNumber(dataConnector)) {
                    return false;
                }

                return dataConnector < value;
            },
        },
        {
            code: '<=',
            title: '<=',
            isValue: true,
            valueType: 'number',
            evaluate(dataConnector, value) {
                if (!_isNumber(dataConnector)) {
                    return false;
                }

                return dataConnector <= value;
            },
        },
        {
            code: '=',
            title: '=',
            isValue: true,
            valueType: 'number',
            evaluate(dataConnector, value) {
                if (!_isNumber(dataConnector)) {
                    return false;
                }

                return dataConnector === value;
            },
        },
        {
            code: '>=',
            title: '>=',
            isValue: true,
            valueType: 'number',
            evaluate(dataConnector, value) {
                if (!_isNumber(dataConnector)) {
                    return false;
                }

                return dataConnector >= value;
            },
        },
        {
            code: '>',
            title: '>',
            isValue: true,
            valueType: 'number',
            evaluate(dataConnector, value) {
                if (!_isNumber(dataConnector)) {
                    return false;
                }

                return dataConnector > value;
            },
        },
    ],

    string: [
        {
            code: 'defined',
            title: 'should be defined',
            evaluate(dataConnector, value) {
                if (!_isString(dataConnector)) {
                    return false;
                }

                return dataConnector.length > 0;
            },
        },
        {
            code: 'undefined',
            title: 'should be undefined',
            evaluate(dataConnector, value) {
                if (!_isString(dataConnector)) {
                    return true;
                }

                return dataConnector.length === 0;
            },
        },
        {
            code: 'includes',
            title: 'includes',
            isValue: true,
            valueType: 'string',
            evaluate(dataConnector, value) {
                if (!_isString(dataConnector)) {
                    return false;
                }

                return dataConnector.includes(value);
            },
        },
        {
            code: 'doesntInclude',
            title: 'doesn\'t include',
            isValue: true,
            valueType: 'string',
            evaluate(dataConnector, value) {
                if (!_isString(dataConnector)) {
                    return true;
                }

                return !dataConnector.includes(value);
            },
        },
        {
            code: 'startsWith',
            title: 'starts with',
            isValue: true,
            valueType: 'string',
            evaluate(dataConnector, value) {
                if (!_isString(dataConnector)) {
                    return false;
                }

                const regex = new RegExp('^' + _escapeRegExp(value), 'i');
                return regex.test(dataConnector);
            },
        },
        {
            code: 'endsWith',
            title: 'ends with',
            isValue: true,
            valueType: 'string',
            evaluate(dataConnector, value) {
                if (!_isString(dataConnector)) {
                    return false;
                }

                const regex = new RegExp(_escapeRegExp(value) + '$', 'i');
                return regex.test(dataConnector);
            },
        },
        {
            code: 'regex',
            title: 'regex',
            isValue: true,
            valueType: 'string',
            evaluate(dataConnector, value) {
                if (!_isString(dataConnector)) {
                    return false;
                }

                const regex = new RegExp(value, 'i');
                return regex.test(dataConnector);
            },
        },
    ],

    array: [
        {
            code: 'includes',
            title: 'includes',
            isValue: true,
            valueType: 'string',
            evaluate(dataConnector, value) {
                if (!_isArray(dataConnector)) {
                    return false;
                }

                return dataConnector.includes(value);
            },
        },
        {
            code: 'doesntInclude',
            title: 'doesn\'t include',
            isValue: true,
            valueType: 'string',
            evaluate(dataConnector, value) {
                if (!_isArray(dataConnector)) {
                    return true;
                }

                return !dataConnector.includes(value);
            },
        },
    ],
};
export default operators;


export const evaluateCondition = ({ dataConnector, type, operator, value }) => {
    const evaluate = _find(operators[type], { code: operator }).evaluate;

    return evaluate(dataConnector, value);
};
