import React, { PropTypes } from 'react';
import { reduxForm }  from 'redux-form';
import _isEqual from 'lodash/isEqual';
import _find from 'lodash/find';
import FormBlock from '../../components/FormBlock';
import FieldBlock from '../../components/FieldBlock';
import domOnlyProps from 'cccisd-redux-form-helpers/helpers/domOnlyProps.js';
import ShowConditionsBlock, { validate as showConditionsValidate } from './ShowConditionsBlock';

import commonStyle from '../../styles/common.css';


export default ({
    fields = [],
    validate = () => {},
    initialValues = {},
    withTemplate = false,
    withAccommodations = false,
    withBlock = false,
    withShowConditions = false,
}) => {
    const getFields = () => {
        var formFields = fields;

        if (withTemplate) {
            formFields = [
                ...formFields,
                'template',
                'templateSettings',
                'isTemplateExpanded',
            ];
        }
        if (withAccommodations) {
            formFields = [
                ...formFields,
                'isAccommodationsExpanded',
                'overrideAccommodations',
                'accommodations.masking',
                'accommodations.highlighting',
                'accommodations.autoplayQuestions',
                'accommodations.autoplayChoices',
            ];
        }
        if (withBlock) {
            formFields = [
                ...formFields,
                'blockIsSeparatePage',
                'blockPlayerTemplate',
                'blockPlayerTemplateSettings',
                'blockIsPlayerTemplateExpanded',
            ];
        }
        if (withShowConditions) {
            formFields = [
                ...formFields,
                'showConditionsExpanded',
                'showConditionsRules[].id',
                'showConditionsRules[].dataConnector',
                'showConditionsRules[].operator',
                'showConditionsRules[].value',
            ];
        }

        return formFields;
    };

    const getInitialValues = () => {
        var formInitialValues = initialValues;

        if (withTemplate) {
            formInitialValues = {
                ...formInitialValues,
                template: '',
                templateSettings: {},
                isTemplateExpanded: false,
            };
        }

        return formInitialValues;
    };


    const getValidate = (values) => {
        let errors = validate(values) || {};

        if (withShowConditions) {
            errors = { ...errors, ...showConditionsValidate(values) };
        }

        return errors;
    };


    return (WrappedComponent) => {
        const Form = React.createClass({
            propTypes: {
                fields: PropTypes.object.isRequired,
                valid: PropTypes.bool,
                values: PropTypes.object,
                updatePreview: PropTypes.func,
                showPreviewError: PropTypes.func,
                availableTemplates: PropTypes.array,
                availablePlayerTemplates: PropTypes.array,
                formBlockType: PropTypes.string,
                children: PropTypes.array,
                DataConnectorsPicker: PropTypes.func,
            },
            getDefaultProps() {
                return {
                    updatePreview() {},
                    showPreviewError() {},
                    formBlockType: 'primary',
                    availableTemplates: [],
                    availablePlayerTemplates: [],
                };
            },
            componentDidUpdate(prevProps) {
                const { valid, values } = this.props;

                if (_isEqual(prevProps.valid, valid) && _isEqual(prevProps.values, values)) {
                    return;
                }

                if (!valid) {
                    this.props.showPreviewError();
                } else {
                    this.props.updatePreview(values);
                }
            },
            _onTemplateChange(event) {
                const { template, templateSettings } = this.props.fields;
                const newTemplateName = event.target.value;

                const newTemplate = _find(this.props.availableTemplates, { name: newTemplateName });
                const newTemplateSettings = newTemplate ? newTemplate.defaultSettings : {};

                template.onChange(newTemplateName);
                templateSettings.onChange(newTemplateSettings);
            },
            _renderTemplateBlock() {
                const {
                    template,
                    templateSettings,
                    isTemplateExpanded,
                } = this.props.fields;

                var templateSettingsComponent = null;
                if (template.value !== '') {
                    const currentTemplate = _find(this.props.availableTemplates, { name: template.value });
                    if (currentTemplate) {
                        const SettingsComponent = currentTemplate.settingsComponent;
                        if (SettingsComponent) {
                            templateSettingsComponent = (
                                <SettingsComponent
                                    initialValues={{ ...currentTemplate.defaultSettings, ...templateSettings.value }}
                                    onChange={templateSettings.onChange}
                                    settings={this.props.values}
                                    children={this.props.children}
                                    DataConnectorsPicker={this.props.DataConnectorsPicker}
                                />
                            );
                        }
                    }
                }


                return (
                    <FormBlock
                        title="Template"
                        type={this.props.formBlockType}
                        show={isTemplateExpanded.value}
                        handleClick={show => isTemplateExpanded.onChange(show)}
                    >
                        {this.props.availableTemplates.length === 0 ? (
                            <div>There are no templates available</div>
                        ) : (
                            <div>
                                <div className={!templateSettingsComponent && commonStyle.lastRow}>
                                    <FieldBlock field={template} label="Template" comment="Some templates could override element settings">
                                        <select
                                            {...domOnlyProps(template)}
                                            className="form-control"
                                            style={{ width: 'auto' }}
                                            onChange={this._onTemplateChange}
                                        >
                                            <option value="">-- Default --</option>
                                            {this.props.availableTemplates.map(item => (
                                                <option value={item.name} key={item.name}>{item.title}</option>
                                            ))}
                                        </select>
                                    </FieldBlock>
                                </div>
                                {templateSettingsComponent}
                            </div>
                        )}
                    </FormBlock>
                );
            },
            _changeOverrideAccommodations(event) {
                const {
                    overrideAccommodations,
                    accommodations: {
                        masking,
                        highlighting,
                        autoplayQuestions,
                        autoplayChoices,
                    },
                } = this.props.fields;

                const list = [
                    masking,
                    highlighting,
                    autoplayQuestions,
                    autoplayChoices,
                ];

                overrideAccommodations.onChange(event);
                list.forEach(item => !item.value && item.onChange(false));
            },
            _renderAccommodationsBlock() {
                const {
                    isAccommodationsExpanded,
                    overrideAccommodations,
                    accommodations: {
                        masking,
                        highlighting,
                        autoplayQuestions,
                        autoplayChoices,
                    },
                } = this.props.fields;

                const list = [
                    { field: masking, label: 'Masking' },
                    { field: highlighting, label: 'Highlighting' },
                    { field: autoplayQuestions, label: 'Autoplay questions' },
                    { field: autoplayChoices, label: 'Autoplay choices' },
                ];

                return (
                    <FormBlock
                        title="Accommodations"
                        type={this.props.formBlockType}
                        show={isAccommodationsExpanded.value}
                        handleClick={show => isAccommodationsExpanded.onChange(show)}
                    >
                        <div className="checkbox" style={{ margin: '0' }}>
                            <label>
                                <input
                                    type="checkbox"
                                    {...domOnlyProps(overrideAccommodations)}
                                    onChange={this._changeOverrideAccommodations}
                                />
                                {' '}
                                Override accommodations
                            </label>
                        </div>

                        {overrideAccommodations.value && (
                            <div className={commonStyle.lastRow}>
                                <FieldBlock field={masking} label="New accommodations" style={{ marginTop: '1em' }}>
                                    {list.map(item => (
                                        <div key={item.field.name} className="checkbox" style={{ margin: '0' }}>
                                            <label>
                                                <input type="checkbox" {...domOnlyProps(item.field)} /> {item.label}
                                            </label>
                                        </div>
                                    ))}
                                </FieldBlock>
                            </div>
                        )}
                    </FormBlock>
                );
            },
            _onPlayerTemplateChange(event) {
                const {
                    blockIsSeparatePage,
                    blockPlayerTemplate,
                    blockPlayerTemplateSettings,
                } = this.props.fields;
                const newTemplateName = event.target.value;

                const newTemplate = _find(this.props.availablePlayerTemplates, { name: newTemplateName });
                const newTemplateSettings = newTemplate ? newTemplate.defaultSettings : {};

                blockPlayerTemplate.onChange(newTemplateName);
                blockPlayerTemplateSettings.onChange(newTemplateSettings);
                blockIsSeparatePage.onChange(true);
            },
            _renderBlockBlock() {
                const {
                    blockIsSeparatePage,
                    blockPlayerTemplate,
                    blockPlayerTemplateSettings,
                    blockIsPlayerTemplateExpanded,
                } = this.props.fields;

                const availableTemplates = this.props.availablePlayerTemplates;
                if (availableTemplates.length === 0) {
                    return null;
                }

                var templateSettingsComponent = null;
                const currentTemplate = _find(availableTemplates, { name: blockPlayerTemplate.value });
                if (currentTemplate && currentTemplate.settingsComponent) {
                    const SettingsComponent = currentTemplate.settingsComponent;
                    templateSettingsComponent = (
                        <SettingsComponent
                            initialValues={{ ...currentTemplate.defaultSettings, ...blockPlayerTemplateSettings.value }}
                            onChange={blockPlayerTemplateSettings.onChange}
                        />
                    );
                }

                return (
                    <div>
                        <div className="checkbox" style={{ marginTop: '0' }}>
                            <label>
                                <input
                                    type="checkbox"
                                    {...blockIsSeparatePage}
                                    disabled={blockPlayerTemplate.value !== ''}
                                /> put children to the separate page?
                            </label>
                        </div>

                        <FormBlock
                            title="Override Player Template"
                            type="primary"
                            show={blockIsPlayerTemplateExpanded.value}
                            handleClick={show => blockIsPlayerTemplateExpanded.onChange(show)}
                        >
                            <div className={!templateSettingsComponent && commonStyle.lastRow}>
                                <FieldBlock field={blockPlayerTemplate} label="Player template for this block">
                                    <select
                                        {...blockPlayerTemplate}
                                        className="form-control"
                                        style={{ width: 'auto' }}
                                        onChange={this._onPlayerTemplateChange}
                                    >
                                        <option value="">-- inherited --</option>
                                        {availableTemplates.map(item => (
                                            <option value={item.name} key={item.name}>{item.title}</option>
                                        ))}
                                    </select>
                                </FieldBlock>
                            </div>
                            {templateSettingsComponent}
                        </FormBlock>
                    </div>
                );
            },
            render() {
                return (
                    <div>
                        <WrappedComponent {...this.props} />
                        {withTemplate && this._renderTemplateBlock()}
                        {withAccommodations && this._renderAccommodationsBlock()}
                        {withBlock && this._renderBlockBlock()}
                        {withShowConditions && (
                            <ShowConditionsBlock
                                fields={this.props.fields}
                                formBlockType={this.props.formBlockType}
                                DataConnectorsPicker={this.props.DataConnectorsPicker}
                            />
                        )}
                    </div>
                );
            },
        });

        return reduxForm({
            form: 'builderForm',
            fields: getFields(),
            validate: getValidate,
            initialValues: getInitialValues(),
        })(Form);
    };
};
